package org.apache.beehive.controls.runtime.packaging;

import java.beans.EventSetDescriptor;
import java.beans.IntrospectionException;
import java.beans.MethodDescriptor;
import java.lang.ref.Reference;
import java.lang.ref.SoftReference;
import java.lang.reflect.Method;

/**
 * The ControlEventSetDescriptor is a result of an infortunate evoluntary flaw in the 
 * java.beans.EventSetDescriptor class.  The getListeners functionality for event sets was
 * added after the initial implementation, and unfortunately, there is no constructor that
 * let you specify <b>both</b> the MethodDescriptors for events <b>and</b> the getListener
 * method.  To compensate for this, we must subclass and provide our own getGetListenerMethod
 * implementation.
 */
public class ControlEventSetDescriptor extends EventSetDescriptor
{
    /**
     * This constructor adds the getListenerMethod argument that is missing from the JDK!
     */
    public ControlEventSetDescriptor(String eventSetName, Class<?> listenerType,
                                     MethodDescriptor[] listenerMethodDescriptors,
                                     Method addListenerMethod, Method removeListenerMethod,
                                     Method getListenerMethod)
           throws IntrospectionException
    {
        super(eventSetName, listenerType, listenerMethodDescriptors, 
              addListenerMethod, removeListenerMethod);

        // Follow the same pattern as the JDK and store the method as a soft reference, so
        // the introspector (alone) won't prevent Class garbage collection.
        _getMethodRef = new SoftReference<Method>(getListenerMethod);
    }

    /**
     * Override the default implementation of getGetListenerMethod to return the method
     * provided in the constructor.
     */
    public Method getGetListenerMethod()
    {
        if (_getMethodRef == null)
            return null;

        return _getMethodRef.get();
    }

    Reference<Method> _getMethodRef;
}
