/**
 Copyright 2004 The Apache Software Foundation.

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

 $Header:$
 */
package org.apache.beehive.controls.system.ejb.internal;

import java.util.List;

import org.apache.beehive.controls.system.ejb.EJBInfo;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 *
 */
public class EJBJarDescriptorHandler {

    public static EJBJarDescriptorHandler getInstance() {
        return new EJBJarDescriptorHandler();
    }

    private EJBJarDescriptorHandler() {}

    public void assemble(Document document, EJBInfo ejbInfo, String ejbLinkValue) {
        Element ejbJarType = document.getDocumentElement();
        Element entBeansType = DomUtils.getChildElementByName(ejbJarType, "enterprise-beans");

        // TODO: support Entity & other bean types
        List sessionArray = DomUtils.getChildElementsByName(entBeansType, "session");
        for ( Object e : sessionArray )
        {
            Element sessionBean = (Element)e;

            if (ejbInfo.isLocal()) {
                insertEJBLocalRefInEJBJar(sessionBean, ejbInfo, ejbLinkValue, document);
            }
            else {
                insertEJBRefInEJBJar(sessionBean, ejbInfo, ejbLinkValue, document);
            }
        }
    }

    private void insertEJBRefInEJBJar(Element sessionBean, EJBInfo ei, String ejbLinkValue, Document ejbDoc)
    {
        System.err.println("EJBControlAssembler.insertEJBRefInEJBJar() called");
        System.err.println("ejbLinkValue =" + ejbLinkValue);
        List ejbRefArray = DomUtils.getChildElementsByName(sessionBean, "ejb-ref");
        String insertedEjbRefName = ei.getRefName();

        Node nextSibling = null;
        for (int j = ejbRefArray.size() - 1; j >= 0; j--) {
            Element ejbRef = (Element) ejbRefArray.get(j);
            String ejbRefName = DomUtils.getChildElementText(ejbRef, "ejb-ref-name");
            if (insertedEjbRefName.equals(ejbRefName)) {
                nextSibling = ejbRef.getNextSibling();
                sessionBean.removeChild(ejbRef);
                break;
            }
        }

        // insert a new <ejb-ref> entry and fill in the values
        Element insertedEjbRef = ejbDoc.createElement("ejb-ref");
        if (nextSibling != null) {
            sessionBean.insertBefore(insertedEjbRef, nextSibling);
        } else {
            sessionBean.insertBefore(insertedEjbRef, findEjbRefInsertPoint(sessionBean));
        }

        Element ejbRefName = ejbDoc.createElement("ejb-ref-name");
        ejbRefName.setTextContent(insertedEjbRefName);
        insertedEjbRef.appendChild(ejbRefName);

        Element ejbRefType = ejbDoc.createElement("ejb-ref-type");
        ejbRefType.setTextContent(ei.getBeanType());
        insertedEjbRef.appendChild(ejbRefType);

        Element homeType = ejbDoc.createElement("home");
        homeType.setTextContent(ei.getHomeInterface().getName());
        insertedEjbRef.appendChild(homeType);

        Element remoteType = ejbDoc.createElement("remote");
        remoteType.setTextContent(ei.getBeanInterface().getName());
        insertedEjbRef.appendChild(remoteType);

        Element ejbLink = ejbDoc.createElement("ejb-link");
        ejbLink.setTextContent(ejbLinkValue);
        insertedEjbRef.appendChild(ejbLink);
    }

    private void insertEJBLocalRefInEJBJar(Element sessionBean, EJBInfo ei, String ejbLinkValue, Document ejbDoc)
    {
        System.err.println("EJBControlAssembler.insertEJBLocalRefInEJBJar() called");
        System.err.println("ejbLinkValue =" + ejbLinkValue);
        List ejbLocalRefArray = DomUtils.getChildElementsByName(sessionBean, "ejb-local-ref");
        String insertedEjbRefName = ei.getRefName();

        Node nextSibling = null;
        for (int j = ejbLocalRefArray.size() - 1; j >= 0; j--) {
            Element ejbLocalRef = (Element) ejbLocalRefArray.get(j);
            String ejbRefName = DomUtils.getChildElementText(ejbLocalRef, "ejb-ref-name");
            if (insertedEjbRefName.equals(ejbRefName)) {
                nextSibling = ejbLocalRef.getNextSibling();
                sessionBean.removeChild(ejbLocalRef);
                break;
            }
        }

        // insert a new <ejb-local-ref> entry and fill in the values
        Element insertedEjbLocalRef = ejbDoc.createElement("ejb-local-ref");
        if (nextSibling != null) {
            sessionBean.insertBefore(insertedEjbLocalRef, nextSibling);
        }
        else {
            sessionBean.insertBefore(insertedEjbLocalRef, findEjbLocalRefInsertPoint(sessionBean));
        }

        Element ejbRefName = ejbDoc.createElement("ejb-ref-name");
        ejbRefName.setTextContent(insertedEjbRefName);
        insertedEjbLocalRef.appendChild(ejbRefName);

        Element ejbRefType = ejbDoc.createElement("ejb-ref-type");
        ejbRefType.setTextContent(ei.getBeanType());
        insertedEjbLocalRef.appendChild(ejbRefType);

        Element homeType = ejbDoc.createElement("local-home");
        homeType.setTextContent(ei.getHomeInterface().getName());
        insertedEjbLocalRef.appendChild(homeType);

        Element localType = ejbDoc.createElement("local");
        localType.setTextContent(ei.getBeanInterface().getName());
        insertedEjbLocalRef.appendChild(localType);

        Element ejbLink = ejbDoc.createElement("ejb-link");
        ejbLink.setTextContent(ejbLinkValue);
        insertedEjbLocalRef.appendChild(ejbLink);
    }

    private Node findEjbRefInsertPoint(Element parent) {

        Element e =  DomUtils.getChildElementByName(parent, "ejb-local-ref");
        if (e != null)
           return e;

        return findEjbLocalRefInsertPoint(parent);
    }

    private Node findEjbLocalRefInsertPoint(Element parent) {

        Element e = DomUtils.getChildElementByName(parent, "service-ref");
        if (e != null)
            return e;

        e = DomUtils.getChildElementByName(parent, "resource-ref");
        if (e != null)
            return e;

        e = DomUtils.getChildElementByName(parent, "resource-env-ref");
        if (e != null)
            return e;

        e = DomUtils.getChildElementByName(parent, "message-destination-ref");
        if(e != null)
            return e;

        e = DomUtils.getChildElementByName(parent, "security-role-ref");
        if (e != null)
            return e;

        e = DomUtils.getChildElementByName(parent, "security-identity");
        if (e != null)
            return e;

        return null;
    }
}
